	---------------------------------------------------
	--       Linear Programming Simplex Method       --
	-- Gerardo V. Lozada, M.S., P.E.E. - 10 May 2003 --
	---------------------------------------------------


include get_enh.e
include wildcard.e
include graphics.e

procedure prtab(integer nr,integer nc,sequence a,sequence b,sequence nb,integer fno)
	sequence idum
	for i=1 to nr do
		if (i<nr) then
			printf(1,"X%d: ",{nb[i]})
			printf(fno,"X%d: ",{nb[i]})
		else
			printf(1,"Obj: ",{})
			printf(fno,"Obj: ",{})
		end if
		for j=1 to nc do
			printf(1,"%10.3e ",{a[i][j]})
			printf(fno,"%10.3e ",{a[i][j]})
		end for
		printf(1,"    RHS = %10.3e\n",{b[i]})
		printf(fno,"    RHS = %10.3e\n",{b[i]})
	end for
	printf(1,"\n",{})
	printf(fno,"\n",{})
	printf(1,"\nPress any key to continue: ",{})
	idum=getstr(0)
end procedure

procedure simplex(integer nv,integer nl,integer ng,integer ne,sequence a,sequence b,sequence nb,integer min_prob,integer fno)
	integer nc,nvc,nr,ipiv,jpiv,jj,found
	atom c,amin
	nc=nl+ng+ne
	nr=nc+1
	nvc=nv+nl+2*ng+ne
	printf(1,"\nTableu after adjusting Object Function:\n\n",{})
	printf(fno,"\nTableu after adjusting Object Function:\n\n",{})
	prtab(nr,nvc,a,b,nb,fno)
	jpiv=1
	while (jpiv>0) do
		jpiv=0
		amin=0.0
		for j=1 to nvc do
			if (a[nr][j]<amin) then
				jpiv=j
				amin=a[nr][j]
			end if
		end for
		if (jpiv>0) then
			ipiv=0
			amin=1.0e99
			for i=1 to nc do
				if (a[i][jpiv]>0.0) then
					if (b[i]/a[i][jpiv]<amin) then
						ipiv=i
						amin=b[i]/a[i][jpiv]
					end if
				end if
			end for
			if (ipiv>0) then
				nb[ipiv]=jpiv
				printf(1,"\nPivoting on Row %d, Column %d:\n\n",{ipiv,jpiv})
				printf(fno,"\nPivoting on Row %d, Column %d:\n\n",{ipiv,jpiv})
				c=1.0/a[ipiv][jpiv]
				for j=1 to nvc do
					a[ipiv][j]*=c
				end for
				b[ipiv]*=c
				for i=1 to nr do
					if (i!=ipiv) then
						c=a[i][jpiv]
						for j=1 to nvc do
							a[i][j]-=c*a[ipiv][j]
						end for
							b[i]-=c*b[ipiv]
					end if
				end for
				prtab(nr,nvc,a,b,nb,fno)
			else
				printf(1,"Unbounded case\n",{})
				printf(fno,"Unbounded case\n",{})
				abort(1)
			end if
		end if
	end while
	printf(1,"\nSolution:\n\n",{})
	printf(fno,"\nSolution:\n\n",{})
	for i=1 to nvc do
		jj=1
		found=0
		printf(1,"X%d = ",{i})
		printf(fno,"X%d = ",{i})
		while ((jj<=nc)and(found=0)) do
			if (nb[jj]=i) then
				found=1
				printf(1,"%10.3e\n",{b[jj]})
				printf(fno,"%10.3e\n",{b[jj]})
			end if
			jj+=1
		end while
		if (found=0) then
			printf(1,"0.0\n",{})
			printf(fno,"0.0\n",{})
		end if
	end for
	if (min_prob) then
		b[nr]*=-1.0
		printf(1,"\nMinimum ",{})
		printf(fno,"\nMinimum ",{})
	else
		printf(1,"\nMaximum ",{})
		printf(fno,"\nMaximum ",{})
	end if
	printf(1,"Object Function Value: %10.3e\n\n",{b[nr]})
	printf(fno,"Object Function Value: %10.3e\n\n",{b[nr]})
end procedure

sequence filename,ifilename,ofilename,title,nb,a,b
integer idum,fni,fno,nv,nc,nr,nvc,nl,ng,ne,min_prob
atom t
idum=text_rows(25)
text_color(BLUE)
bk_color(CYAN)
clear_screen()
printf(1,"Linear Programming Primal Simplex Maximization/Minimization Method\n",{})
printf(1,"(c) Gerardo V. Lozada, M.S., P.E.E. - 10 May 2003\n\n",{})
printf(1,"Input Data File Name (no extension, .DAT automatically appended): ",{})
filename=getstr(0)
ifilename=filename&".dat"
ofilename=filename&".out"
fni=open(ifilename,"r")
fno=open(ofilename,"w")
title=getstr(fni)
printf(1,"\nCase Title: %s\n\n",{title})
printf(1,"\nInput Filename is %s\n\n",{upper(ifilename)})
printf(fno,"Linear Programming Primal Simplex Maximization/Minimization Method\n",{})
printf(fno,"(c) Gerardo V. Lozada, M.S., P.E.E. - 10 May 2003\n\n",{})
printf(fno,"\nInput Filename is %s\n\n",{upper(ifilename)})
printf(fno,"\nCase Title: %s\n\n",{title})
nv=input(fni)
nl=input(fni)
ng=input(fni)
ne=input(fni)
nc=nl+ng+ne
nr=nc+1
nvc=nv+nl+2*ng+ne
t=0.0
if ((ng+ne)>0) then
	t=100.0
end if
a=repeat(repeat(0.0,nvc),nr)
b=repeat(0.0,nr)
nb=repeat(0,nc)
-- Read Constraints
printf(1,"\nConstraints:\n\n",{})
printf(fno,"\nConstraints:\n\n",{})
for i=1 to nr-1 do
	for j=1 to nv do
		a[i][j]=input(fni)
		printf(1," ",{})
		printf(fno," ",{})
		if ((j>1)and(a[i][j]>=0.0)) then
			printf(1,"+",{})
			printf(fno,"+",{})
		end if
		printf(1,"%10.3e*X%d",{a[i][j],j})
		printf(fno,"%10.3e*X%d",{a[i][j],j})
	end for
	if (i<=nl) then
		a[i][nv+i]=1.0
		nb[i]=nv+i
		printf(1," <= ",{})
		printf(fno," <= ",{})
	else
		if (i<=nl+ng) then
			a[i][nv+nl+(i-nl)*2-1]=-1.0
			a[i][nv+nl+(i-nl)*2]=1.0
			a[nr][nv+nl+(i-nl)*2]=t
			nb[i]=nv+nl+(i-nl)*2
			printf(1," => ",{})
			printf(fno," => ",{})
		else
			a[i][nv+nl+ng*2+(i-nl-ng)]=1.0
			a[nr][nv+nl+ng*2+(i-nl-ng)]=t
			nb[i]=nv+nl+ng*2+(i-nl-ng)
			printf(1," = ",{})
			printf(fno," = ",{})
		end if
	end if
	b[i]=input(fni)
	printf(1,"%10.3e\n",{b[i]})
	printf(fno,"%10.3e\n",{b[i]})
end for
-- Read Object Function to be Maximized/Minimized
printf(1,"\nObjective Function:\n\n",{})
printf(fno,"\nObjective Function:\n\n",{})
min_prob=0
for j=1 to nv do
	a[nr][j]=input(fni)
	if (a[nr][j]<0.0) then
		min_prob=1
	end if
	printf(1," ",{})
	printf(fno," ",{})
	if ((j>1)and(a[nr][j]>=0.0)) then
		printf(1,"+",{})
		printf(fno,"+",{})
	end if
	printf(1,"%10.3e*X%d",{a[nr][j],j})
	printf(fno,"%10.3e*X%d",{a[nr][j],j})
	a[nr][j]*=-1.0
end for
printf(1," to be ",{})
printf(fno," to be ",{})
if (min_prob) then
	printf(1,"Minimized\n",{})
	printf(fno,"Minimized\n",{})
else
	printf(1,"Maximized\n",{})
	printf(fno,"Maximized\n",{})
end if
printf(1,"\nInitial Tableu:\n",{})
printf(fno,"\nInitial Tableu:\n",{})
prtab(nr,nvc,a,b,nb,fno)
if (ng+ne>0) then
	for i=nl+1 to nr-1 do
		for j=1 to nvc do
			a[nr][j]-=t*a[i][j]
		end for
		b[nr]-=t*b[i]
	end for
end if
close(fni)
simplex(nv,nl,ng,ne,a,b,nb,min_prob,fno)
printf(1,"\n\nOutput Text Data Filename: %s\n",{upper(ofilename)})
printf(fno,"\n\nProgram Execution Terminated.\n",{})
close(fno)
printf(1,"\n\nProgram Execution Terminated.\n",{})
